@@ -1,15 +1,16 @@
 
 /*
  *  agUtils.c
- *  $Id: agUtils.c,v 4.16 2007/08/05 17:09:50 bkorb Exp $
+ *  $Id: agUtils.c,v 4.20 2008/07/28 04:51:29 bkorb Exp $
  *
- *  Time-stamp:        "2007-07-04 11:14:34 bkorb"
- *  Last Committed:    $Date: 2007/08/05 17:09:50 $
+ *  Time-stamp:        "2008-07-26 10:00:15 bkorb"
+ *  Last Committed:    $Date: 2008/07/28 04:51:29 $
  *
  *  This is the main routine for autogen.
  *
  *  This file is part of AutoGen.
- *  AutoGen copyright (c) 1992-2007 by Bruce Korb - all rights reserved
+ *  AutoGen copyright (c) 1992-2008 by Bruce Korb - all rights reserved
+ *  AutoGen copyright (c) 1992-2008 by Bruce Korb - all rights reserved
  *
  * AutoGen is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -367,6 +368,21 @@ skipScheme( tCC* pzSrc,  tCC* pzEnd )
 }
 
 
+LOCAL int
+count_nl(char const * pz)
+{
+    int ct = 0;
+    for (;;) {
+        char const * p = strchr(pz, '\n');
+        if (p == NULL)
+            break;
+        ct++;
+        pz = p + 1;
+    }
+    return ct;
+}
+
+
 LOCAL tCC*
 skipExpression( tCC* pzSrc, size_t len )
 {
@@ -374,7 +390,7 @@ skipExpression( tCC* pzSrc, size_t len )
 
  guess_again:
 
-    while (isspace( *pzSrc )) pzSrc++;
+    while (IS_WHITESPACE_CHAR(*pzSrc)) pzSrc++;
     if (pzSrc >= pzEnd)
         return pzEnd;
     switch (*pzSrc) {
@@ -397,7 +413,7 @@ skipExpression( tCC* pzSrc, size_t len )
         break;
     }
 
-    while (! isspace( *pzSrc ))  pzSrc++;
+    while (! IS_WHITESPACE_CHAR(*pzSrc))  pzSrc++;
     return (pzSrc > pzEnd) ? pzEnd : pzSrc;
 }
 /*
