@@ -1,14 +1,15 @@
 /*
  *  expFormat.c
  *
- *  Time-stamp:        "2007-07-04 11:18:40 bkorb"
- *  Last Committed:    $Date: 2007/07/04 20:51:11 $
+ *  Time-stamp:        "2007-11-19 21:12:49 bkorb"
+ *  Last Committed:    $Date: 2008/01/23 00:36:01 $
  *
- *  $Id: expFormat.c,v 4.22 2007/07/04 20:51:11 bkorb Exp $
+ *  $Id: expFormat.c,v 4.26 2008/01/23 00:36:01 bkorb Exp $
  *  This module implements formatting expression functions.
  *
  *  This file is part of AutoGen.
- *  AutoGen copyright (c) 1992-2007 by Bruce Korb - all rights reserved
+ *  AutoGen copyright (c) 1992-2008 by Bruce Korb - all rights reserved
+ *  AutoGen copyright (c) 1992-2008 by Bruce Korb - all rights reserved
  *
  * AutoGen is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -36,6 +37,18 @@ static char const zGpl[] =
 "%2$sYou should have received a copy of the GNU General Public License along\n"
 "%2$swith this program.  If not, see <http://www.gnu.org/licenses/>.";
 
+static char const zAgpl[] =
+"%2$s%1$s is free software: you can redistribute it and/or modify it\n"
+"%2$sunder the terms of the GNU General Public License as published by the\n"
+"%2$sFree Software Foundation, either version 3 of the License, or\n"
+"%2$s(at your option) any later version.\n%2$s\n"
+"%2$s%1$s is distributed in the hope that it will be useful, but\n"
+"%2$sWITHOUT ANY WARRANTY; without even the implied warranty of\n"
+"%2$sMERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.\n"
+"%2$sSee the GNU Affero General Public License for more details.\n%2$s\n"
+"%2$sYou should have received a copy of the GNU Affero General Public License\n"
+"%2$salong with this program.  If not, see <http://www.gnu.org/licenses/>.";
+
 static char const zLgpl[] =
 "%2$s%1$s is free software: you can redistribute it and/or modify it\n"
 "%2$sunder the terms of the GNU Lesser General Public License as published\n"
@@ -270,7 +283,7 @@ ag_scm_error( SCM res )
     case GH_TYPE_STRING:
         pzMsg  = ag_scm2zchars( res, "error string" );
         msgLen = AG_SCM_STRLEN( res );
-        while (isspace( *pzMsg ) && (--msgLen > 0)) pzMsg++;
+        while (IS_WHITESPACE_CHAR(*pzMsg) && (--msgLen > 0)) pzMsg++;
 
         /*
          *  IF the message starts with the number zero,
@@ -279,7 +292,7 @@ ag_scm_error( SCM res )
          */
         if (msgLen <= 0)
             abrt = PROBLEM;
-        else if (isdigit( *pzMsg ) && (strtol( pzMsg, NULL, 0 ) == 0))
+        else if (IS_DEC_DIGIT_CHAR(*pzMsg) && (strtol(pzMsg, NULL, 0) == 0))
             abrt = PROBLEM;
         break;
 
@@ -322,7 +335,6 @@ ag_scm_error( SCM res )
  *  @code{prefix} contains the string to start each output line, and
  *  @code{prog_name} contains the name of the program the copyright is
  *  about.
- *
 =*/
 SCM
 ag_scm_gpl( SCM prog_name, SCM prefix )
@@ -354,6 +366,52 @@ ag_scm_gpl( SCM prog_name, SCM prefix )
 }
 
 
+/*=gfunc agpl
+ *
+ * what:  GNU Affero General Public License
+ * general_use:
+ *
+ * exparg: prog-name, name of the program under the GPL
+ * exparg: prefix, String for starting each output line
+ *
+ * doc:
+ *
+ *  Emit a string that contains the GNU Affero General Public License.
+ *  It takes two arguments:
+ *  @code{prefix} contains the string to start each output line, and
+ *  @code{prog_name} contains the name of the program the copyright is
+ *  about.
+=*/
+SCM
+ag_scm_agpl( SCM prog_name, SCM prefix )
+{
+    char*   pzPfx  = ag_scm2zchars( prefix, "AGPL line prefix" );
+    char*   pzPrg  = ag_scm2zchars( prog_name, "program name" );
+    char*   pzRes;
+    SCM     res;
+
+    /*
+     *  Get the addresses of the program name and prefix strings.
+     *  Make sure they are reasonably sized (<256 for program name
+     *  and <128 for a line prefix).  Copy them to the scratch buffer.
+     */
+    if (AG_SCM_STRLEN( prog_name ) >= 256)
+        AG_ABEND( aprf( zPfxMsg, zProgLen, 256 ));
+
+    if (AG_SCM_STRLEN( prefix ) >= 128)
+        AG_ABEND( aprf( zPfxMsg, zPfxLen, 128 ));
+
+    /*
+     *  Allocate-sprintf the result string, then put it in a new SCM.
+     */
+    pzRes = aprf( zAgpl, pzPrg, pzPfx );
+
+    res = AG_SCM_STR02SCM( pzRes );
+    AGFREE( (void*)pzRes );
+    return res;
+}
+
+
 /*=gfunc lgpl
  *
  * what:  GNU Library General Public License
@@ -527,7 +585,7 @@ ag_scm_license( SCM license, SCM prog_name, SCM owner, SCM prefix )
      */
     {
         char* pz = (char*)lic.mi.txt_data + lic.mi.txt_size;
-        while (isspace( pz[-1] ) && (pz > (char*)lic.mi.txt_data))
+        while (IS_WHITESPACE_CHAR(pz[-1]) && (pz > (char*)lic.mi.txt_data))
             pz--;
         *pz = NUL;
     }
