@@ -1,10 +1,10 @@
 
 /*
  *  expString.c
- *  $Id: expString.c,v 4.23 2008/01/23 00:36:01 bkorb Exp $
+ *  $Id: expString.c,v 4.25 2008/11/16 23:56:59 bkorb Exp $
  *
- *  Time-stamp:        "2007-07-06 13:34:10 bkorb"
- *  Last Committed:    $Date: 2008/01/23 00:36:01 $
+ *  Time-stamp:        "2008-11-16 12:28:30 bkorb"
+ *  Last Committed:    $Date: 2008/11/16 23:56:59 $
  *
  *  This module implements expression functions that
  *  manipulate string values.
@@ -1189,6 +1189,37 @@ ag_scm_string_substitute( SCM Str, SCM Match, SCM Repl )
     return res;
 }
 
+
+/*=gfunc time_string_to_number
+ *
+ * what:   duration string to seconds
+ * general_use:
+ * exparg: time_spec, string to parse
+ *
+ * doc:    Convert the argument string to a time period in seconds.
+ *         The string may use multiple parts consisting of days, hours
+ *         minutes and seconds.  These are indicated with a suffix of
+ *         @code{d}, @code{h}, @code{m} and @code{s} respectively.
+ *         Hours, minutes and seconds may also be represented with
+ *         @code{HH:MM:SS} or, without hours, as @code{MM:SS}.
+=*/
+SCM
+ag_scm_time_string_to_number( SCM time_spec )
+{
+    extern time_t parse_duration(char const * in_pz);
+
+    tCC *   pz;
+    time_t  time_period;
+
+    if (! AG_SCM_STRING_P( time_spec ))
+        return SCM_UNDEFINED;
+
+    pz = AG_SCM_CHARS( time_spec );
+    time_period = parse_duration(pz);
+    
+    return AG_SCM_INT2SCM((int)time_period);
+}
+
 /*
  * Local Variables:
  * mode: C
