@@ -1,14 +1,15 @@
 
 /*
- *  $Id: funcEval.c,v 4.18 2007/08/05 17:09:51 bkorb Exp $
+ *  $Id: funcEval.c,v 4.21 2008/01/23 00:36:02 bkorb Exp $
  *
- *  Time-stamp:        "2007-07-04 11:26:16 bkorb"
- *  Last Committed:    $Date: 2007/08/05 17:09:51 $
+ *  Time-stamp:        "2007-11-12 20:41:59 bkorb"
+ *  Last Committed:    $Date: 2008/01/23 00:36:02 $
  *
  *  This module evaluates macro expressions.
  *
  *  This file is part of AutoGen.
- *  AutoGen copyright (c) 1992-2007 by Bruce Korb - all rights reserved
+ *  AutoGen copyright (c) 1992-2008 by Bruce Korb - all rights reserved
+ *  AutoGen copyright (c) 1992-2008 by Bruce Korb - all rights reserved
  *
  * AutoGen is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -544,7 +545,7 @@ mLoad_Expr( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
             if (pzNextExpr >= pz + srcLen)
                 AG_ABEND_IN( pT, pMac, "`?' needs two expressions" );
 
-            if (! isspace( *pzNextExpr ))
+            if (! IS_WHITESPACE_CHAR(*pzNextExpr))
                 AG_ABEND_IN( pT, pMac, "No space between expressions" );
 
             /*
@@ -553,7 +554,7 @@ mLoad_Expr( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
              *  into the macro type code as the "secondary type".
              */
             *(pzNextExpr++) = NUL;
-            while (isspace( *pzNextExpr ))  pzNextExpr++;
+            while (IS_WHITESPACE_CHAR(*pzNextExpr))  pzNextExpr++;
             pMac->res |= (exprType( pzNextExpr ) << EMIT_SECONDARY_SHIFT);
             pMac->endIndex = pzNextExpr - pT->pzTemplText;
         }
