@@ -1,31 +1,28 @@
 
 /*
- *  $Id: funcIf.c,v 4.12 2007/04/28 22:19:23 bkorb Exp $
+ *  $Id: funcIf.c,v 4.17 2008/01/23 00:36:02 bkorb Exp $
  *
- *  Time-stamp:        "2006-09-23 19:51:36 bkorb"
- *  Last Committed:    $Date: 2007/04/28 22:19:23 $
+ *  Time-stamp:        "2007-12-02 15:10:46 bkorb"
+ *  Last Committed:    $Date: 2008/01/23 00:36:02 $
  *
  *  This module implements the _IF text function.
- */
-
-/*
- *  AutoGen copyright 1992-2007 Bruce Korb
  *
- *  AutoGen is free software.
- *  You may redistribute it and/or modify it under the terms of the
- *  GNU General Public License, as published by the Free Software
- *  Foundation; either version 2, or (at your option) any later version.
+ *  This file is part of AutoGen.
+ *  AutoGen copyright (c) 1992-2008 by Bruce Korb - all rights reserved
+ *  AutoGen copyright (c) 1992-2008 by Bruce Korb - all rights reserved
  *
- *  AutoGen is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * AutoGen is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with AutoGen.  See the file "COPYING".  If not,
- *  write to:  The Free Software Foundation, Inc.,
- *             51 Franklin Street, Fifth Floor,
- *             Boston, MA  02110-1301, USA.
+ * AutoGen is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 tSCC zNoIfEnd[]  = "%s ERROR:  cannot find ENDIF\n\t'%s'\n";
@@ -42,7 +39,7 @@ static tIfStack  current_if;
 static tLoadProc mLoad_Elif, mLoad_Else;
 
 /* = = = START-STATIC-FORWARD = = = */
-/* static forward declarations maintained by :mkfwd */
+/* static forward declarations maintained by mk-fwd */
 static ag_bool
 eval_true( void );
 
@@ -71,7 +68,7 @@ eval_true( void )
     ag_bool res = AG_TRUE;
     tCC* pz = evalExpression( &needFree );
 
-    if (isdigit( *pz ))
+    if (IS_DEC_DIGIT_CHAR(*pz))
         res = (atoi(pz) == 0) ? AG_FALSE : AG_TRUE;
 
     else switch (*pz) {
@@ -97,7 +94,7 @@ eval_true( void )
     }
 
     if (needFree)
-        AGFREE( (void*)pz );
+        AGFREE(pz);
 
     return res;
 }
@@ -172,7 +169,7 @@ mFunc_If( tTemplate* pT, tMacro* pMac )
                          ? "ELSE clause" : pT->pzTemplText + pMac->ozText,
                          pMac->lineNo );
 
-                if (OPT_VALUE_TRACE < TRACE_EVERYTHING)
+                if (OPT_VALUE_TRACE == TRACE_EVERYTHING)
                     fprintf( pfTrace, zFileLine, pCurTemplate->pzTplFile,
                              pIf->lineNo );
             }
@@ -187,7 +184,7 @@ mFunc_If( tTemplate* pT, tMacro* pMac )
         fprintf( pfTrace, "IF `%s' macro selected no clause\n",
                  pCurTemplate->pzTemplText + pCurMacro->ozText );
 
-        if (OPT_VALUE_TRACE < TRACE_EVERYTHING)
+        if (OPT_VALUE_TRACE == TRACE_EVERYTHING)
             fprintf( pfTrace, zFileLine, pCurTemplate->pzTplFile,
                      pIf->lineNo );
     }
@@ -256,7 +253,7 @@ mFunc_While( tTemplate* pT, tMacro* pMac )
     if (OPT_VALUE_TRACE >= TRACE_BLOCK_MACROS) {
         fprintf( pfTrace, "WHILE macro repeated %d times\n", ct );
 
-        if (OPT_VALUE_TRACE < TRACE_EVERYTHING)
+        if (OPT_VALUE_TRACE == TRACE_EVERYTHING)
             fprintf( pfTrace, zFileLine, pT->pzTplFile, pMac->lineNo );
     }
 
