@@ -1,31 +1,28 @@
 
 /*
- *  $Id: functions.c,v 4.14 2007/02/04 17:44:12 bkorb Exp $
+ *  $Id: functions.c,v 4.18 2008/01/23 00:36:02 bkorb Exp $
  *
- *  Time-stamp:        "2007-02-03 08:22:04 bkorb"
- *  Last Committed:    $Date: 2007/02/04 17:44:12 $
+ *  Time-stamp:        "2008-01-22 08:46:58 bkorb"
+ *  Last Committed:    $Date: 2008/01/23 00:36:02 $
  *
  *  This module implements text functions.
- */
-
-/*
- *  AutoGen copyright 1992-2007 Bruce Korb
  *
- *  AutoGen is free software.
- *  You may redistribute it and/or modify it under the terms of the
- *  GNU General Public License, as published by the Free Software
- *  Foundation; either version 2, or (at your option) any later version.
+ *  This file is part of AutoGen.
+ *  AutoGen copyright (c) 1992-2008 by Bruce Korb - all rights reserved
+ *  AutoGen copyright (c) 1992-2008 by Bruce Korb - all rights reserved
  *
- *  AutoGen is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ * AutoGen is free software: you can redistribute it and/or modify it
+ * under the terms of the GNU General Public License as published by the
+ * Free Software Foundation, either version 3 of the License, or
+ * (at your option) any later version.
  *
- *  You should have received a copy of the GNU General Public License
- *  along with AutoGen.  See the file "COPYING".  If not,
- *  write to:  The Free Software Foundation, Inc.,
- *             51 Franklin Street, Fifth Floor,
- *             Boston, MA  02110-1301, USA.
+ * AutoGen is distributed in the hope that it will be useful, but
+ * WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+ * See the GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License along
+ * with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 tSCC zCantInc[] = "cannot include file";
@@ -76,11 +73,11 @@ mFunc_Include( tTemplate* pT, tMacro* pMac )
             else *pzE = NUL;
         }
 
-        if (OPT_VALUE_TRACE > TRACE_NOTHING) {
+        if (OPT_VALUE_TRACE > TRACE_DEBUG_MESSAGE) {
             tSCC zTplFmt[] = "Template %s included\n";
             tSCC zLinFmt[] = "\tfrom %s line %d\n";
             fprintf( pfTrace, zTplFmt, pNewTpl->pzTplFile );
-            if (OPT_VALUE_TRACE < TRACE_EVERYTHING)
+            if (OPT_VALUE_TRACE == TRACE_EVERYTHING)
                 fprintf( pfTrace, zLinFmt, pCurTemplate->pzTplFile,
                          pMac->lineNo );
         }
@@ -230,6 +227,10 @@ mFunc_Text( tTemplate* pT, tMacro* pMac )
  *
  *    If the native macro name code is @code{#}, then the
  *    entire macro function is treated as a comment and ignored.
+ *
+ *    @example
+ *    [+ # say what you want, but no '+' before any ']' chars +]
+ *    @end example
 =*/
 tMacro*
 mLoad_Comment( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
@@ -319,14 +320,11 @@ mLoad_Unknown( tTemplate* pT, tMacro* pMac, tCC** ppzScan )
     pzCopy = pT->pNext; /* next text dest   */
     pMac->ozText = (pzCopy - pT->pzTemplText);
     pMac->res    = 0;
-
-    do  {
-        *(pzCopy++) = *(pzSrc++);
-    } while (--srcLen > 0);
-    *(pzCopy++) = NUL;
-    *(pzCopy++) = NUL; /* double terminate */
-
-    pT->pNext = pzCopy;
+    memcpy(pzCopy, pzSrc, srcLen);
+    pzCopy      += srcLen;
+    *(pzCopy++)  = NUL;
+    *pzCopy      = NUL; /* double terminate */
+    pT->pNext    = pzCopy;
 
     return pMac + 1;
 
