@@ -1,24 +1,22 @@
 
-;;;  AutoGen copyright 1992-2006 Bruce Korb
+;;; Time-stamp:        "2007-11-18 10:35:38 bkorb"
+;;; Last Committed:    $Date: 2007/11/18 22:49:17 $
 ;;;
-;;; Time-stamp:        "2006-09-23 19:50:26 bkorb"
-;;; Last Committed:    $Date: 2006/09/24 02:59:00 $
+;;; This file is part of AutoGen.
+;;; AutoGen copyright (c) 1992-2007 by Bruce Korb - all rights reserved
 ;;;
-;;; AutoGen is free software.
-;;; You may redistribute it and/or modify it under the terms of the
-;;; GNU General Public License, as published by the Free Software
-;;; Foundation; either version 2, or (at your option) any later version.
+;;; AutoGen is free software: you can redistribute it and/or modify it
+;;; under the terms of the GNU General Public License as published by the
+;;; Free Software Foundation, either version 3 of the License, or
+;;; (at your option) any later version.
 ;;;
-;;; AutoGen is distributed in the hope that it will be useful,
-;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
-;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-;;; GNU General Public License for more details.
+;;; AutoGen is distributed in the hope that it will be useful, but
+;;; WITHOUT ANY WARRANTY; without even the implied warranty of
+;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+;;; See the GNU General Public License for more details.
 ;;;
-;;; You should have received a copy of the GNU General Public License
-;;; along with AutoGen.  See the file "COPYING".  If not,
-;;; write to:  The Free Software Foundation, Inc.,
-;;;            51 Franklin Street, Fifth Floor,
-;;;            Boston, MA  02110-1301, USA.
+;;; You should have received a copy of the GNU General Public License along
+;;; with this program.  If not, see <http://www.gnu.org/licenses/>.
 ;;;
 ;;; This module defines all the scheme initialization for AutoGen.
 ;;; It gets sucked up into directives.h as a single ANSI-C string.
@@ -96,86 +94,16 @@
 (define tmp-dir "")
 (define make-tmp-dir
   (lambda () 
-    (begin (set! tmp-dir (shell
-  "tmp_dir=`mktemp -d ./.ag-XXXXXX` 2>/dev/null
-  test -d \"${tmp_dir}\" || {
-    tmp_dir=${TMPDIR:-/tmp}/.ag-$$
-    mkdir ${tmp_dir} || die cannot mkdir ${tmp_dir}
-  } ; echo ${tmp_dir}" ))
+    (begin (if (= tmp-dir "") (set! tmp-dir (shell
+  "mk_tmp_dir ; echo ${tmp_dir}" ))))
 
-  (add-cleanup (string-append  "rm -rf " tmp-dir))
+  (add-cleanup (string-append
+   "test \"${VERBOSE:-false}\" = true || rm -rf " tmp-dir))
 
-)))
-
-;;; /*=gfunc   make_header_guard
-;;;  *
-;;;  * what:   make self-inclusion guard
-;;;  *
-;;;  * exparg: name , header group name
-;;;  *
-;;;  * doc:
-;;;  *   This function will create a @code{#ifndef}/@code{#define}
-;;;  *   sequence for protecting a header from multiple evaluation.
-;;;  *   It will also set the Scheme variable @code{header-file}
-;;;  *   to the name of the file being protected and it will set
-;;;  *   @code{header-guard} to the name of the @code{#define} being
-;;;  *   used to protect it.  It is expected that this will be used
-;;;  *   as follows:
-;;;  *   @example
-;;;  *   [+ (make-header-guard "group_name") +]
-;;;  *   ...
-;;;  *   #endif /* [+ (. header-guard) +]
-;;;  *
-;;;  *   #include "[+ (. header-file)  +]"
-;;;  *   @end example
-;;;  *   @noindent
-;;;  *   The @code{#define} name is composed as follows:
-;;;  *
-;;;  *   @enumerate
-;;;  *   @item
-;;;  *   The first element is the string argument and a separating underscore.
-;;;  *   @item
-;;;  *   That is followed by the name of the header file with illegal
-;;;  *   characters mapped to underscores.
-;;;  *   @item
-;;;  *   The end of the name is always, "@code{_GUARD}".
-;;;  *   @item
-;;;  *   Finally, the entire string is mapped to upper case.
-;;;  *   @end enumerate
-;;;  *
-;;;  *   The final @code{#define} name is stored in an SCM symbol named
-;;;  *   @code{header-guard}.  Consequently, the concluding @code{#endif} for the
-;;;  *   file should read something like:
-;;;  *
-;;;  *   @example
-;;;  *   #endif /* [+ (. header-guard) +] */
-;;;  *   @end example
-;;;  *
-;;;  *   The name of the header file (the current output file) is also stored
-;;;  *   in an SCM symbol, @code{header-file}.  Therefore, if you are also
-;;;  *   generating a C file that uses the previously generated header file,
-;;;  *   you can put this into that generated file:
-;;;  *
-;;;  *   @example
-;;;  *   #include "[+ (. header-file) +]"
-;;;  *   @end example
-;;;  *
-;;;  *   Obviously, if you are going to produce more than one header file from
-;;;  *   a particular template, you will need to be careful how these SCM symbols
-;;;  *   get handled.
-;;; =*/
-;;;
-(define header-file  "")
-(define header-guard "")
-(define (make-header-guard hdr-pfx)
-  (begin
-    (set! header-file  (out-name))
-    (set! header-guard (string-upcase! (string->c-name! (string-append
-      (if (string? hdr-pfx) hdr-pfx "HEADER") "_" header-file "_GUARD"
-      ))))
-    (sprintf "#ifndef %1$s\n#define %1$s" header-guard)
 ) )
 
+(define header-file     "")
+(define header-guard    "")
 (define autogen-version "AUTOGEN_VERSION")
 (define c-file-line-fmt "#line %2$d \"%1$s\"\n")
 
@@ -332,6 +260,23 @@
    stt-idx
 )))
 
+;;; /*=gfunc   string_table_add_ref
+;;;  *
+;;;  * what:   Add an entry to a string table, get reference
+;;;  * general-use:
+;;;  *
+;;;  * exparg: st-name , the name of the array of characters
+;;;  * exparg: str-val , the (possibly) new value to add
+;;;  *
+;;;  * doc:    Identical to string-table-add, except the value returned
+;;;  *         is the string "st-name" '+' and the index returned by
+;;;  *         string-table-add.
+;;; =*/
+;;;
+(define string-table-add-ref (lambda (st-name str-val)
+   (string-append st-name "+"
+      (number->string (string-table-add st-name str-val)) ) ))
+
 ;;; /*=gfunc   emit_string_table
 ;;;  *
 ;;;  * what:   output a string table
@@ -362,6 +307,67 @@
    (emit ";\n")
 )))
 
+;;; /*=gfunc   string_table_size
+;;;  *
+;;;  * what:   print the current size of a string table
+;;;  * general-use:
+;;;  *
+;;;  * exparg: st-name , the name of the array of characters
+;;;  *
+;;;  * doc:    Returns the current byte count of the string table.
+;;; =*/
+;;;
+(define string-table-size (lambda (st-name)
+  (hash-ref (hash-ref stt-table st-name) "current-index") ))
+
+;;; /*=gfunc   gperf_code
+;;;  *
+;;;  * what:   emit the source of the generated gperf program
+;;;  * general-use:
+;;;  *
+;;;  * exparg: st-name , the name of the gperf hash list
+;;;  *
+;;;  * doc:
+;;;  *  Returns the contents of the emitted code, suitable
+;;;  *  for inclusion in another program.  The interface contains
+;;;  *  the following elements:
+;;;  *
+;;;  *  @table @samp
+;;;  *  @item struct @i{<st-name>}_index
+;;;  *  containg the fields: @code{@{char const * name, int const id; @};}
+;;;  *
+;;;  *  @item @i{<st-name>}_hash()
+;;;  *  This is the hashing function with local only scope (static).
+;;;  *
+;;;  *  @item @i{<st-name>}_find()
+;;;  *  This is the searching and validation function.  The first argument
+;;;  *  is the string to look up, the second is its length.
+;;;  *  It returns a pointer to the corresponding @code{@i{<st-name>}_index}
+;;;  *  entry.
+;;;  *  @end table
+;;;  *
+;;;  *  Use this in your template as follows where "@i{<st-name>}" was
+;;;  *  set to be "@code{lookup}":
+;;;  *
+;;;  *  @example
+;;;  *  [+ (make-gperf "lookup" (join "\n" (stack "name_list")))
+;;;  *     (gperf-code "lookup") +]
+;;;  *  void my_fun(char * str) @{
+;;;  *  struct lookup_index * li = lookup_find(str, strlen(str));
+;;;  *  if (li != NULL) printf("%s yields %d\n", str, li->idx);
+;;;  *  @end example
+;;; =*/
+;;;
+(define gperf-code (lambda (gp-name) (shellf
+  "sed -e '1,/^#line/d' \
+       -e '/#include/d' \
+       -e '/#line/d' \
+       -e '/^[ \t]*$/d' \
+       -e 's/^const struct /static const struct /' \
+       -e '/^int main(/,$d' ${gpdir}/%s.c"
+  gp-name
+)))
+
 (use-modules (ice-9 debug))
 
 (read-enable 'positions)
