@@ -1,14 +1,15 @@
 
 /*
- *  $Id: tpLoad.c,v 4.25 2007/10/30 22:02:16 bkorb Exp $
+ *  $Id: tpLoad.c,v 4.27 2008/12/29 06:14:24 bkorb Exp $
  *
- * Time-stamp:        "2007-10-30 08:17:58 bkorb"
- * Last Committed:    $Date: 2007/10/30 22:02:16 $
+ * Time-stamp:        "2008-12-25 11:12:48 bkorb"
+ * Last Committed:    $Date: 2008/12/29 06:14:24 $
  *
  *  This module will load a template and return a template structure.
  *
  * This file is part of AutoGen.
- * AutoGen copyright (c) 1992-2007 by Bruce Korb - all rights reserved
+ * AutoGen copyright (c) 1992-2008 by Bruce Korb - all rights reserved
+ * AutoGen copyright (c) 1992-2008 by Bruce Korb - all rights reserved
  *
  * AutoGen is free software: you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the
@@ -430,6 +431,9 @@ unloadTemplate( tTemplate* pT )
 LOCAL void
 cleanup( tTemplate* pTF )
 {
+    if (HAVE_OPT(USED_DEFINES))
+        print_used_defines();
+
     optionFree(&autogenOptions);
 
     for (;;) {
