@@ -1,7 +1,7 @@
 
 /*
- *  $Id: autoopts.c,v 4.35 2008/08/27 14:45:20 bkorb Exp $
- *  Time-stamp:      "2008-08-10 10:17:06 bkorb"
+ *  $Id: autoopts.c,v 4.36 2008/12/20 18:35:26 bkorb Exp $
+ *  Time-stamp:      "2008-12-16 14:52:28 bkorb"
  *
  *  This file contains all of the routines that must be linked into
  *  an executable to use the generated option processing.  The optional
@@ -685,8 +685,8 @@ nextOption( tOptions* pOpts, tOptState* pOptState )
 
         default:
         case TOPT_DEFAULT:
-            fputs( "AutoOpts lib error: defaulted to option with optional arg\n",
-                   stderr );
+            fputs("AutoOpts lib error: defaulted to option with optional arg\n",
+                  stderr );
             exit( EX_SOFTWARE );
         }
 
@@ -830,7 +830,8 @@ doPresets( tOptions* pOpts )
      *  has a --load-opts option.  See if a command line option has disabled
      *  option presetting.
      */
-    if (pOpts->specOptIdx.save_opts != 0) {
+    if (  (pOpts->specOptIdx.save_opts != NO_EQUIVALENT)
+       && (pOpts->specOptIdx.save_opts != 0)) {
         pOD = pOpts->pOptDesc + pOpts->specOptIdx.save_opts + 1;
         if (DISABLED_OPT(pOD))
             return SUCCESS;
@@ -1097,7 +1098,8 @@ optionProcess(
      *  THEN do that now before testing for conflicts.
      *       (conflicts are ignored in preset options)
      */
-    if (pOpts->specOptIdx.save_opts != 0) {
+    if (  (pOpts->specOptIdx.save_opts != NO_EQUIVALENT)
+       && (pOpts->specOptIdx.save_opts != 0)) {
         tOptDesc*  pOD = pOpts->pOptDesc + pOpts->specOptIdx.save_opts;
 
         if (SELECTED_OPT( pOD )) {
